import React from 'react';
import dynamic from 'dva/dynamic';
import { getMenuData } from './menu';

// wrapper of dynamic
const dynamicWrapper = (app, models, component) => dynamic({
  app,
  // eslint-disable-next-line no-underscore-dangle
  models: () => models.filter(m => !app._models.some(({ namespace }) => namespace === m)).map(m => import(`../models/${m}.js`)),
  // add routerData prop
  component: () => {
    const p = component();
    return new Promise((resolve, reject) => {
      p.then((Comp) => {
        resolve(props => <Comp {...props} routerData={getRouterData(app)} />);
      }).catch(err => reject(err));
    });
  },
});

function getFlatMenuData(menus) {
  let keys = {};
  menus.forEach((item) => {
    if (item.children) {
      keys[item.path] = item.name;
      keys = { ...keys, ...getFlatMenuData(item.children) };
    } else {
      keys[item.path] = item.name;
    }
  });
  return keys;
}

export const getRouterData = (app) => {
  const routerData = {
    '/': {
      component: dynamicWrapper(app, ['user', 'login'], () => import('../layouts/BasicLayout')),
    },
    '/resource/gallery': {
      component: dynamicWrapper(app, ['resource'], () => import('../routes/Resource/gallery')),
    },
    '/resource/video': {
      component: dynamicWrapper(app, ['resource'], () => import('../routes/Resource/video')),
    },
    '/terminal/campus': {
      component: dynamicWrapper(app, ['campus'], () => import('../routes/Campus')),
    },
    '/terminal/user': {
      component: dynamicWrapper(app, ['terminal'], () => import('../routes/Terminal')),
    },
    '/merchant': {
      component: dynamicWrapper(app, ['merchant/merchant'], () => import('../routes/Merchant')),
    },
    '/merchant/add': {
      component: dynamicWrapper(app, ['merchant/detail'], () => import('../routes/Merchant/detail')),
      name: '添加厂商',
    },
    '/merchant/edit/:id': {
      component: dynamicWrapper(app, ['merchant/detail'], () => import('../routes/Merchant/detail')),
      name: '修改厂商',
    },
    '/tag/group': {
      component: dynamicWrapper(app, ['group/group', 'merchant/merchant'], () => import('../routes/TagGroup')),
    },
    '/tag/group/add': {
      component: dynamicWrapper(app, ['group/detail', 'merchant/merchant'], () => import('../routes/TagGroup/detail')),
      name: '添加标签组',
    },
    '/tag/group/edit/:id': {
      component: dynamicWrapper(app, ['group/detail', 'merchant/merchant'], () => import('../routes/TagGroup/detail')),
      name: '修改标签组',
    },
    '/tag/tag': {
      component: dynamicWrapper(app, ['tag/tag', 'merchant/merchant'], () => import('../routes/Tag')),
    },
    '/tag/tag/add': {
      component: dynamicWrapper(app, ['tag/detail', 'group/group'], () => import('../routes/Tag/detail')),
      name: '添加标签',
    },
    '/tag/tag/edit/:id': {
      component: dynamicWrapper(app, ['tag/detail', 'group/group'], () => import('../routes/Tag/detail')),
      name: '修改标签',
    },
    '/user': {
      component: dynamicWrapper(app, [], () => import('../layouts/UserLayout')),
    },
    '/user/login': {
      component: dynamicWrapper(app, ['login'], () => import('../routes/Login')),
    },
  };
  // Get name from ./menu.js or just set it in the router data.
  const menuData = getFlatMenuData(getMenuData());
  const routerDataWithName = {};
  Object.keys(routerData).forEach((item) => {
    routerDataWithName[item] = {
      ...routerData[item],
      name: routerData[item].name || menuData[item.replace(/^\//, '')],
    };
  });
  return routerDataWithName;
};
