import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import { cloneDeep } from 'lodash';
import { Modal, Button, Table } from 'antd';
import DataSearch from '../DataSearch';
import SelTable from './SelTable';
import ForSelTable from './ForSelTable';

const ButtonGroup = ButtonGroup;

export default class SelectModal extends PureComponent {
  static propTypes = {
    mode: PropTypes.string,
    rowKeyName: PropTypes.string,

    fsTableDataSource: PropTypes.array,
    fsTableColumns: PropTypes.array,
    fstablePagination: PropTypes.oneOfType([PropTypes.bool, PropTypes.object]),
    fsTableLoading: PropTypes.bool,
    onChange: PropTypes.func,

    tableDataSource: PropTypes.array,
    tableColumns: PropTypes.array,
    tablePagination: PropTypes.oneOfType([PropTypes.bool, PropTypes.object]),
    operSort: PropTypes.bool,
    operDel: PropTypes.bool,

    searchProps: PropTypes.object,
  };

  constructor(props) {
    super(props);
    this.state = {
      tableTabLeft: true,
      selTableData: this.props.tableDataSource,
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({ selTableData: nextProps.tableDataSource });
  }

  handleTableTabChange = () => {
    const { tableTabLeft } = this.state;
    this.setState({ tableTabLeft: !tableTabLeft });
  }

  // 向上调整一个位置
  handleOnUp = (key) => {
    const { selTableData } = this.state;
    const { rowKeyName } = this.props;
    const arr = cloneDeep(selTableData);

    // 找到该行在列表中的索引
    const index = arr.findIndex(item => item[rowKeyName] === key);
    //第一个元素或者未找到元素不做操作
    if (!index || -1 === index) return;
    //与前一个元素进行位置互换
    arr.splice(index, 1, ...arr.splice(index - 1, 1, arr[index]));
    // 将sort字段重新赋值
    arr.map((item, index) => item.sort !== undefined ? item.sort = index + 1 : null);

    this.setState({ selTableData: arr });
  }

  // 向下调整一个位置
  handleOnDown = (key) => {
    const { selTableData } = this.state;
    const { rowKeyName } = this.props;
    const arr = cloneDeep(selTableData);

    // 找到该行在列表中的索引
    const index = arr.findIndex(item => item[rowKeyName] == key);
    //最后一个元素或者未找到元素不做操作
    if (index + 1 === arr.length || -1 === index) return;
    //与后一个元素进行位置互换
    arr.splice(index, 1, ...arr.splice(index + 1, 1, arr[index]));
    // 将sort字段重新赋值
    arr.map((item, index) => item.sort !== undefined ? item.sort = index + 1 : null);

    this.setState({ selTableData: arr });
  }

  handleOnAdd = (id) => {

  }

  handleOnDel = (id) => {

  }

  handleOnCancel = () => {
    const { onCancel } = this.props;
    onCancel();
  }

  handleOnOk = () => {
    const { selTableData } = this.state;
    const { onOk } = this.props;
    onOk(selTableData);
  }

  comboConverter = () => {
    const { tableTabLeft, selTableData } = this.state;
    const {
      mode, rowKeyName, searchProps,
      tableColumns, tablePagination, tableDataSource, operSort, operDel,
      fsTableLoading, fsTableColumns, fsTableDataSource, fstablePagination,
      onCancel, onOk, ...modalProps
    } = this.props;

    switch (mode) {
      case 'sort':
        return (
          <Modal
            onCancel={this.handleOnCancel}
            onOk={this.handleOnOk}
            {...modalProps}
          >
            <SelTable
              dataSource={selTableData}
              columns={tableColumns}
              pagination={tablePagination}
              rowKeyName={rowKeyName}
              operSort={operSort}
              operDel={operDel}
              onUp={this.handleOnUp}
              onDown={this.handleOnDown}
            />
          </Modal>
        );
        break;
      // case 'single':
      //   return (
      //     <Modal {...modalProps}>
      //       <DataSearch {...searchProps} />
      //       <ForSelTable {...forSelTableProps}/>
      //     </Modal>
      //   );
      //   break;
      // case 'multiple':
      //   return (
      //     <Modal {...modalProps}>
      //       <DataSearch />
      //       <ButtonGroup>
      //         <Button type="primary" onClick={this.handleTabChange} ghost={tableTabLeft ? false : true}>待选</Button>
      //         <Button type="primary" onClick={this.handleTabChange} ghost={tableTabLeft ? true : false}>{`已选[]`}</Button>
      //       </ButtonGroup>
      //       {tableTabLeft ? <ForSelTable {...forSelTableProps} /> : <SelTable {...selTableProps}/>}
      //     </Modal>
      //   );
      //   break;
      default:
        break;
    }
  }

  render() {
    return this.comboConverter();
  }
}
