import { queryOne, create, update } from '../../services/tag';
import { message } from 'antd';
import pathToRegexp from 'path-to-regexp';
import { Codes } from '../../utils/config';

export default {
  namespace: 'tagDetail',

  state: {
    filters: {},
    operType: 'create',
    currentItem: {},
    modalVisible: false,
    itemLoading: false,
  },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen(({ pathname, state, ...rest }) => {
        const match = pathToRegexp('/tag/tag/edit/:id').exec(pathname);
        if (match) {
          dispatch({ type: 'query', payload: { id: match[1] } });
          dispatch({ type: 'saveFilters', payload: state });
          dispatch({ type: 'saveOperType', payload: { operType: 'update' } });
        }
        if (pathname === '/tag/tag/add') {
          dispatch({ type: 'saveFilters', payload: state });
        }
      });
    }
  },

  effects: {
    * query ({ payload }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { itemLoading: true } });
      const { data, success } = yield call(queryOne, payload);
      if (success) {
        yield put({ type: 'querySuccess', payload: { ...data } });
      }
      yield put({ type: 'changeLoading', payload: { itemLoading: false } });
    },
    * create ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(create, payload);
      if (success) {
        message.success('创建成功!');
        yield put({ type: 'clearPage' });
        if (callback) {
          callback();
        }
      } else {
        message.error('创建失败!');
      }
    },
    * update ({ payload, callback }, { call, put }) {
      const { data, success } = yield call(update, payload);
      if (success) {
        message.success('更新成功!');
        yield put({ type: 'clearPage' });
        if (callback) {
          callback();
        }
      } else {
        message.error('更新失败!');
      }
    },
  },

  reducers: {
    changeLoading(state, { payload }) {
      return { ...state, ...payload };
    },

    querySuccess(state, { payload }) {
      return { ...state, currentItem: payload };
    },

    saveFilters(state, { payload: filters }) {
      return { ...state, filters };
    },

    showModal(state) {
      return { ...state, modalVisible: true };
    },

    hideModal(state) {
      return { ...state, modalVisible: false };
    },

    saveOperType(state, { payload }) {
      return { ...state, ...payload };
    },

    saveSortResult(state, { payload: { itemList } }) {
      const currentItem = { ...state.currentItem, itemList };
      return { ...state, modalVisible: false, currentItem };
    },

    clearPage(state) {
      return { ...state, currentItem: {}, itemLoading: false };
    }
  }
}
